import pytest
from .. import src
import os


@pytest.fixture
def logger():
    logger = src.logger(name='test.log', level=src.Levels.DEBUG)
    yield logger


def test_file_creation(logger):
    logger.debug('Test file creation')
    assert os.path.exists('test.log')


def test_correct_level(logger):
    test_msg = 'TEST'
    logger.debug(test_msg)
    with open('test.log', 'r') as f:
        # The last item is an empty string
        last_record = f.read().split('\n')[-2]
        msg = last_record.split(' - ')[-1]
    assert test_msg == msg



from .baseclass import Database
from .msdatabase import MsSQLDatabase
from .mariadatabase import MariaDatabase

def create_db_manager(manager: str, **kwargs) -> Database:
    managers = {
        'MS': MsSQLDatabase(**kwargs),
        'Maria': MariaDatabase(**kwargs)
    }
    if manager not in managers:
        raise ValueError(f"No factory is implemented for provided value {manager}")
    return managers[manager]




from .defautCredential import AzureBlobManagementDefaultCredential
from .connString import AzureBlobManagementConnectionString
from .baseclass import AzureBlobManager


def create_blob_manager(manager: str, storage_account_name: str) -> AzureBlobManager:
    managers = {
        'defaultCredential': AzureBlobManagementDefaultCredential(storage_account_name),
        'connectionString': AzureBlobManagementConnectionString(storage_account_name)
    }
    if manager not in managers:
        raise ValueError(f"No factory is implemented for provided value {manager}")
    return managers[manager]



import pytest
from .. import src
from ..src.databases.baseclass import Database
from ..src.databases.manager import create_db_manager
import os
import pymssql
from sqlalchemy.pool import NullPool
import mariadb

mssql_dict = {
    'server':os.getenv('DB_mssql_SERVER'),
    'user':os.getenv('DB_mssql_USER'),
    'password':os.getenv('DB_mssql_PASSWORD'),
    'database':os.getenv('DB_mssql_DATABASE'),
    'pool_size':NullPool}

maria_dict = {
    'server':os.getenv('DB_mariadb_SERVER'),
    'user':os.getenv('DB_mariadb_USER'),
    'password':os.getenv('DB_mariadb_PASSWORD'),
    'database':os.getenv('DB_mariadb_DATABASE'),
    'pool_size':NullPool}

#manager test
@pytest.mark.parametrize(
    "test_name, init_dict,_class",
    [
        ('MS',mssql_dict, src.MsSQLDatabase),
        ('Maria', maria_dict, src.MariaDatabase)
    ])
def test_correct_manager(test_name, init_dict, _class):

    manager: Database = create_db_manager(test_name, **init_dict)

    assert isinstance(manager, _class)


@pytest.fixture
def mssql_db():
    db = src.MsSQLDatabase(**mssql_dict)
    yield db


@pytest.fixture
def mssql_db_error():
    db_error = src.MsSQLDatabase(
            server=os.getenv('DB_mssql_SERVER'),
            port=os.getenv('DB_mssql_PORT'),
            user=os.getenv('DB_mssql_USER_ERROR'),
            password=os.getenv('DB_mssql_PASSWORD_ERROR'),
            database=os.getenv('DB_mssql_DATABASE'))
    yield db_error


@pytest.fixture
def maria_db():
    db = src.MariaDatabase(**maria_dict)
    yield db


@pytest.fixture
def maria_db_error():
    db_error = src.MariaDatabase(
            server=os.getenv('DB_mariadb_SERVER'),
            user=os.getenv('DB_mariadb_USER_ERROR'),
            password=os.getenv('DB_mariadb_PASSWORD_ERROR'),
            database=os.getenv('DB_mariadb_DATABASE'))
    yield db_error



def test_mssql_db_error(mssql_db_error):
    with pytest.raises(TypeError):
        with mssql_db_error as conn:
            conn.connected


def test_maria_db_error(maria_db_error):
    with pytest.raises(mariadb.Error):
        with maria_db_error as conn:
            conn.connected



def test_mssql_db_success(mssql_db):
    assert mssql_db.connected


def test_maria_db_success(maria_db):
    with maria_db as conn:
        assert conn.connected

@pytest.mark.skip('Not implemented')
def test_maria_db_select(maria_db):
    with maria_db as conn:
        query = "SELECT * FROM tests.test WHERE id = ?"
        vals = (1,)
        res = conn.select(query=query, vals=vals)
        assert res == [{'id': 1, 'test_name': 'test1'}]

@pytest.mark.skip('Not implemented')
def test_maria_db_insert(maria_db):
    with maria_db as conn:
        query = """INSERT INTO tests.test(test_name) VALUES(?)"""
        data = ('test',)
        res = conn.insert(query=query, data=data)
        assert res == 1



from ..src import create_blob_manager
from ..src.blob.connString import AzureBlobManagementConnectionString
from ..src.blob.defautCredential import AzureBlobManagementDefaultCredential
from ..src.blob.baseclass import AzureBlobManager
import pytest


@pytest.mark.parametrize(
    "test_name,_class",
    [
        ('defaultCredential', AzureBlobManagementDefaultCredential),
        ('connectionString', AzureBlobManagementConnectionString)
    ])
def test_correct_manager(test_name, _class):

    manager: AzureBlobManager = create_blob_manager(test_name, storage_account_name='servoerpblobtest')

    assert isinstance(manager, _class)


@pytest.mark.parametrize(
    "test_name,_class",
    [
        ('defaultCredential', AzureBlobManagementDefaultCredential)
    ])
def test_connection(test_name, _class):

    manager: AzureBlobManager = create_blob_manager(test_name, storage_account_name='servoerpblobtest')
    manager._generate_blob_service_client_()
    assert manager.blob_service_client.api_version != ''


from .baseclass import Database
from marioutils.databasemanagement.src import exceptions
import sqlalchemy
import mariadb

class MariaDatabase(Database):
    def __init__(
            self,
            server: str,
            user: str,
            password: str,
            database: str = None,
            as_dict: bool = True,
            port: int = 3306,
            pool_size: int | sqlalchemy.pool.NullPool = 5,
            max_overflow: int = 10):
        super().__init__(
            server,
            user,
            password,
            database,
            as_dict,
            port,
            pool_size,
            max_overflow)
        self.create_pool()

    def get_connection(self):
        c = mariadb.connect(
            user=self.user,
            password=self.password,
            host=self.server,
            port=self.port,
            database=self.database
        )
        return c

    def get_cursor(self):
        if self._connection is None:
            raise exceptions.ConnectionException
        if self._cursor is None:
            self._cursor = self._connection.cursor()
        else:
            return self._cursor

    def fetchall(self):
        try:
            res = self._cursor.fetchall()
            if not self.as_dict:
                return res
            else:
                columns = [a[0] for a in self._cursor.description]
                results = [{c: v for c, v in zip(columns, r)} for r in res]
                return results
        except Exception as e:
            return e




from .baseclass import Database
import pymssql
from marioutils.databasemanagement.src import exceptions
import sqlalchemy


class MsSQLDatabase(Database):

    def __init__(
            self,
            server: str,
            user: str,
            password: str,
            database: str = None,
            as_dict: bool = True,
            port: int = 1433,
            pool_size: int | sqlalchemy.pool.NullPool = 5,
            max_overflow: int = 10) -> pymssql.Connection:
        super().__init__(
            server,
            user,
            password,
            database,
            as_dict,
            pool_size,
            max_overflow)
        self.create_pool()

    def get_connection(self):
        c = pymssql.connect(
            server=self.server,
            user=self.user,
            password=self.password,
            database=self.database,
            as_dict=self.as_dict,
            login_timeout=20,
            timeout=20
        )
        return c

    def get_cursor(self):
        if self._connection is None:
            raise exceptions.ConnectionException
        if self._cursor is None:
            self._cursor = self.connection.cursor()
